@@ -1,10 +1,9 @@
 package DBIx::Class::Fixtures::External::File;
-$DBIx::Class::Fixtures::External::File::VERSION = '1.001027_02';
+
 use strict;
 use warnings;
 
-use File::Spec::Functions 'catfile', 'splitpath';
-use File::Path 'mkpath';
+use File::Spec::Functions 'catfile';
 
 sub _load {
   my ($class, $path) = @_;
@@ -31,8 +30,6 @@ sub backup {
 sub restore {
   my ($class, $key, $content, $args) = @_;
   my $path = catfile($args->{path}, $key);
-  my ($vol, $directory, $file) = splitpath($path);
-  mkpath($directory) unless -d $directory;
   $class->_save($path, $content);
 }
 
